<?php

/*
Plugin Name: Cnhk Slideshow
Description: Cnhk Slideshow is a simple jQuery slideshow plugin with a drag and drop system for editing slides order. Images files are stored in the plugin directory ( Not mixed with Wordpress's Media Library ).
Version: 1.0
Author: Rija Rajaonah
License: GLPv2 or later
*/

session_start();

$cnhk_token_snap = md5( uniqid( rand (), true ) );
$_SESSION['cnhk']['token_snap'] = $cnhk_token_snap;

register_activation_hook( __FILE__, 'cnhk_install' );
register_deactivation_hook( __FILE__, 'cnhk_uninstall' );

define( 'DS', DIRECTORY_SEPARATOR );
define( 'CNHK_DIR', dirname( realpath( __FILE__ ) ) );
define( 'CNHK_ADMIN_SLIDE', admin_url('admin.php?page=cnhk-slide') );
define( 'CNHK_ADMIN_MAIN', admin_url('admin.php?page=cnhk_mainpage') );
define( 'CNHK_ADMIN_SETTINGS', admin_url('admin.php?page=cnhk-settings') );
define( 'CNHK_URL', plugin_dir_url(__FILE__) );

add_action( 'admin_menu', 'cnhk_admin_menu' );
add_action( 'admin_init', 'cnhk_init' );
add_action( 'admin_notices', 'cnhk_admin_note' );
add_action( 'admin_enqueue_scripts', 'cnhk_admin_script' );
add_action( 'wp_enqueue_scripts', 'cnhk_front_script' );
add_action( 'contextual_help', 'cnhk_help', 10, 3 );
add_action( 'pre_get_posts', 'cnhk_redirect' );
add_action( 'widgets_init', 'cnhk_widget' );
add_filter( 'plugin_action_links', 'cnhk_add_link', 10, 2 );

/*
*Add setting link in the plugin list
*/

function cnhk_add_link( $lnk, $file ) {
	static $cnhk_this_plugin;
	if ( ! isset( $cnhk_this_plugin ) ) $cnhk_this_plugin = plugin_basename( __FILE__ );
	if ( $file == $cnhk_this_plugin ) {
		$settings_link = '<a href="admin.php?page=cnhk-settings">Settings</a>';
		array_unshift($lnk, $settings_link);
	}
	return $lnk;
}

/*
*Avoid re-sending form after refreshing a page
*/

function cnhk_redirect() {
    if ( ! empty( $_POST['token_del_ss'] ) || ! empty( $_POST['cnhk_token_setting'] ) || ! empty( $_POST['cnhk_token_new_ss'] ) || ! empty( $_POST['cnhk_token_update'] ) || ! empty( $_POST['cnhk_token_delete'] ) || ! empty( $_POST['cnhk_token_ss'] ) ) {
		$_SESSION['cnhk']['savepost'] = $_POST;
		
		$running = $_SERVER['PHP_SELF'];
		if ( ! empty( $_SERVER['QUERY_STRING'] ) )
		{
			$running .= '?' . $_SERVER['QUERY_STRING'];
		}
		wp_redirect( $running );
		die();
	}
	
	if ( isset( $_POST['cnhk_token_file'] ) && trim( $_POST['cnhk_token_file'] ) == $_SESSION['cnhk']['token_file'] ) {
		$referer = admin_url ('admin.php?page=cnhk-slide');
		if ( isset ( $_SERVER['HTTP_REFERER'] ) && $_SERVER['HTTP_REFERER'] == $referer ) {
			$has_error = false;
			if ( $_FILES['cnhk_file']['error'] > 0 ) {
				$_SESSION['cnhk']['upload_error_upload'] = 'err';
				$has_error = true;
			}
			
			$ext = strtolower( substr( strrchr( $_FILES['cnhk_file']['name'], '.' )  ,1 ) );
			
			if ( $_FILES['cnhk_file']['type'] != 'image/jpeg' ) {
				$_SESSION['cnhk']['upload_error_extension'] = 'err';
				$has_error = true;
			}
			
			if ( 2*1024*1024 < $_FILES['cnhk_file']['size'] ) {
				$_SESSION['cnhk']['upload_error_size'] = 'err';
				$has_error = true;
			}
			
			if ( $has_error == false ) {
				$nb = get_file_number();
				$res = CNHK_DIR . DS .'uploads'. DS .'slide'.$nb.'.jpg';
				
				if ( move_uploaded_file( $_FILES['cnhk_file']['tmp_name'], $res ) ) {
					$_SESSION['cnhk']['upload_error_upload'] = '';
					$_SESSION['cnhk']['upload_error_extension'] = '';
					$_SESSION['cnhk']['upload_error_size'] = '';
					$opt = get_option( 'cnhk_options_slides' );
					array_push( $opt[0], "slide{$nb}.jpg" );
					array_push( $opt[1], -1 );
					array_push( $opt[2], "slide{$nb}" );
					array_push( $opt[3], rawurlencode("No link") );
					array_push( $opt[4], time() );
					update_option( 'cnhk_options_slides', $opt );
				}
			}
			
			$running = $_SERVER['PHP_SELF'];
			if ( ! empty( $_SERVER['QUERY_STRING'] ) )
			{
				$running .= '?' . $_SERVER['QUERY_STRING'];
			}
			wp_redirect( $running );
			die();		
		}
	}
	
	if ( isset( $_SESSION['cnhk']['savepost'] ) )
	{
		$_POST = $_SESSION['cnhk']['savepost'];		
		unset( $_SESSION['cnhk']['savepost']);
	}
}

function get_file_number() {
	$i = 1;
	while ( file_exists( CNHK_DIR . DS . "uploads". DS . "slide{$i}.jpg" ) ) {
		$i++;
	}
	return $i;
}

/*
*Contextual Help
*/

function cnhk_help( $help, $screenid, $screen ) {
	
	if ( 'toplevel_page_cnhk_mainpage' == $screen->id ) {
		$help = "<h4>Slideshow :</h4>";
		$help .= "Just drag and drop snapshots to put or reoder slides within the slideshow. ";
		$help .= "You can also drag a slide out of the slideshow, without permanently deleting it.<br />";
	}
	if ( 'slideshow_page_cnhk-settings' == $screen->id ) {
		$help = "<h4>Adding new slideshow</h4>";
		$help .= "Each slideshow name must be unique(case sensitive)<br />";
		$help .= "<h4>Selecting slideshow</h4>";
		$help .= "This dropdown list contains all existing slideshow. It allows you to select a slideshow to edit (or delete). When you delete a slideshow, all slides used in it will be set as unused.<br />";
		$help .= "<h4>Settings</h4>";
		$help .= "<ul>";
		$help .= "<li>By default, all slideshow has a fluid width. It means that the slideshow will take all available width and is resized each time the page is resized (even partially) ";
		$help .= "in accordance with the <em>width/height 's ratio</em> that you defined in this screen. ";
		$help .= "In other terms, all images that you use in this slideshow will be stretched to fit this ratio.</li>";
		$help .= "<li>If you choose the fix width, the slideshow will be wrapped inside a container with the fixed with that you define. ";
		$help .= "You can also choose an alignment for this container to make the slideshow look better when inserted in a post for example.</li>";
		$help .= "</ul>";
		$help .= "<h4>Displaying slideshow</h4>";
		$help .= "You can display slideshows by:";
		$help .= "<ul>";
		$help .= "<li><h5>Using widget</h5>";
		$help .= "Go to the widget menu. Add a <em>cnhk slideshow</em> widget to a sidebar. Then select one of the slideshow you've created and save changes.</li>";
		$help .= "<li><h5>Using a shortcode</h5>";
		$help .= "In your post you can use a shortcode like <code>[cnhk_slideshow name='my_slideshow' width='300' align='float-left']</code> where :<ul>";
		$help .= "<li><strong><em>my_slideshow</em></strong> is the <em>name parameter</em> that you defined on the settings page.</li>";
		$help .= "<li><strong><em>300</em></strong> is the fixed width in pixels of the container (if you want it to be a fixed width slideshow).";
		$help .= " If set, this value will override the one you've entered in the settings page for this slideshow. Then a fluid width slideshow will have locally a fixed width</li>";
		$help .= "<li><strong><em>align-left</em></strong> is the alignment of the container (only for fixed width). If set, this value will override the one you've entered in the settings page. ";
		$help .= "<br />Possible values : <strong>'left', 'right', 'center', 'float-left', 'float-right'.</strong></li></ul><strong><em>Only the name parameter is required.</em></strong></li>";
		$help .= "<li><h5>Using template tag</h5>";
		$help .= "Directly in your theme's files with <code>&lt;?php if ( function_exists( 'cnhk_slideshow' ) ) cnhk_slideshow( 'my_slideshow', 300, 'align-left' ); ?&gt;</code><br />";
		$help .= "Meanings of all parameters are the same as for shortcode and <strong><em>only the name parameter is required.</em></strong> Pay attention on the width parameter which is passed ";
		$help .= "as an integer to the function (no wrapping quotes).</li></ul>";
	}
	if ( 'slideshow_page_cnhk-slide' == $screen->id ) {
		$help = "<h4>Uploading :</h4>";
		$help .= "All uploaded images stay inside the plugin's directory, and will not mix with images in the Wordpress Media Library.<br />";
		$help .= "<h4>Deleting :</h4>";
		$help .= "This will delete the image file (from the plugin's directory) and the corresponding data in Wordpress's database.<br />";
		$help .= "If you mean to reuse the slide prefer to just drop it out of the slide in the <a href='" . CNHK_ADMIN_MAIN . "'><em>\" slideshow \"</em></a> screen<br />";
		$help .= "<h4>Editing :</h4>";
		$help .= "You can use each slide in only one slideshow.";
	}
	return $help;
}

/*
*Add stylesheet and scripts for the front end
*/

function cnhk_front_script() {
	wp_register_style( 'cnhk_front_css', plugins_url( 'css/cnhk-front.css', __FILE__ ) );
	wp_enqueue_style( 'cnhk_front_css' );
	wp_enqueue_script( 'jquery' );
	wp_enqueue_script( 'jquery-cycle', plugins_url( 'js/jquery.cycle.all.js', __FILE__ ), array( 'jquery' ) );
	wp_enqueue_script( 'cnhk_front_js', plugins_url( 'js/cnhk-front.js', __FILE__ ), array( 'jquery-cycle' ) );
}

/*
*Add stylesheets and scripts for admin pages
*/

function cnhk_admin_script() {
	global $pagenow;
	if ( 'admin.php' == $pagenow ) {
	$page_list = array( 'cnhk_mainpage', 'cnhk-slide', 'cnhk-settings' );
		if ( isset( $_GET['page'] ) && in_array( $_GET['page'], $page_list )  ) {
			wp_register_style( 'cnhk-slideshow_css', plugins_url( 'css/cnhk-ss.css', __FILE__ ) );
			wp_enqueue_style( 'cnhk-slideshow_css' );
			wp_enqueue_script( 'jquery' );
			wp_enqueue_script( 'jquery-ui-core' );
			wp_enqueue_script( 'jquery-ui-widget' );
			wp_enqueue_script( 'jquery-ui-mouse' );
			wp_enqueue_script( 'jquery-ui-sortable' );
			wp_enqueue_script( 'cnhk_admin_js', plugins_url( 'js/cnhk-admin.js', __FILE__ ), array(  'jquery', 'jquery-ui-sortable' ) );
		}
	}
}

/*
*Initialisation
*/

function cnhk_init() {
	cnhk_redirect();
}

/*
*Display admin setting error
*/

function cnhk_admin_note() {
	settings_errors();
}

/*
*Activation
*/

function cnhk_install() {
	cnhk_default();
}

/*
*Deactivation
*/

function cnhk_uninstall() {
	unregister_setting( 'cnhk_options_group', 'cnhk_options', 'cnhk_validate' );
	delete_option( 'cnhk_options' );
	$opt = get_option( 'cnhk_options_slides' );
	$size = count( $opt[0] );
	$base = CNHK_DIR . DS . "uploads". DS ;
	for ( $i = 0; $i < $size; $i++ ) {
		unlink($base. $opt[0][$i]);
	}
	delete_option( 'cnhk_options_slides' );
	delete_option( 'cnhk_options_ss' );
}

/*
*Default values for options
*/

function cnhk_default(){
	
	$name				= array();
	$effect				= array();
	$transition_speed	= array();
	$slide_duration		= array();
	$height				= array();
	$width				= array();
	$nav				= array();
	$fixed				= array();
	$fixed_width		= array();
	$fixed_align		= array();
	
	$image 	= array();
	$use	= array();
	$title 	= array();
	$link 	= array();
	$date 	= array();
	
	$arr = array( $name, $effect, $transition_speed, $slide_duration, $height, $width, $nav, $fixed, $fixed_width, $fixed_align );
	$arr2 	= array( $image, $use, $title, $link, $date );
	$ss 	= array( 'ss' => array(), 'elem' => array() );
	
	update_option( 'cnhk_options', $arr );
	update_option( 'cnhk_options_slides', $arr2 );
	update_option( 'cnhk_options_ss', $ss );
}

/*
*Creates the admin menu
*/

function cnhk_admin_menu() {

add_menu_page( 'Slideshow', 'Slideshow', 'manage_options', 'cnhk_mainpage', 'cnhk_main_menu_fn', '', 14 );
add_submenu_page( 'cnhk_mainpage', 'Settings', 'Settings', 'manage_options', 'cnhk-settings', 'cnhk_setting_fn' );
add_submenu_page( 'cnhk_mainpage', 'Slides', 'Slides', 'manage_options', 'cnhk-slide', 'cnhk_slide_fn' );

}

/*
*Display "Slideshow" page in admin panel
*/

function cnhk_main_menu_fn() {
	include_once( 'cnhk-slideshow.php' );
}

/*
*Display "Slides" page in admin panel
*/

function cnhk_slide_fn() {
	if ( ! current_user_can( 'manage_options' ) ) {
		wp_die( 'You do not have sufficient permissions to access this page.' );
	}
	include_once( 'cnhk-slides.php' );
}

/*
*Display slideshow in the front end
*/

function cnhk_slideshow( $instance, $width = null, $align = null ) {
	$index;
	$fixed_width = $width;
	$fixed_align = $align;
	$ss = get_option( 'cnhk_options_ss' );
	$setting = get_option( 'cnhk_options' );
	if ( count( $ss['ss'] ) > 0 ) {
		if ( is_string( $instance ) ) {
			$index = array_search( $instance, $ss['ss'] );
		} else {
				$index = false;
		}
	
		if ( is_int( $index ) && $setting[7][$index] ) {
			if ( ! is_int( $fixed_width ) ) {
				$fixed_width = $setting[8][$index];
			}
			if ( ! is_string( $fixed_align ) ) {
				$fixed_align = $setting[9][$index];
			}
		}
		
		include ( 'cnhk-front.php' );
	}
}

/*
* Display "Settings" page
*/

function cnhk_setting_fn() {
	if ( ! current_user_can( 'manage_options' ) ) {
		wp_die( 'You do not have sufficient permissions to access this page.' );
	}
	require_once( 'settings.php' );
}

/*
* Creates widget
*/

function cnhk_widget() {
	register_widget( 'cnhk_ss_widget' );
}

class Cnhk_Ss_Widget extends WP_Widget {

	function Cnhk_Ss_Widget() {
		$widget_ops = array( 'classname' => 'cnhk-ss-widget', 'description'	=> 'Use this widget to add one slideshow to one of your sidebar.'	);		
		$control_ops = array( 'id_base' => 'cnhk-ss-widget' );		
		$this->WP_Widget( 'cnhk-ss-widget', 'Cnhk Slideshow', $widget_ops, $control_ops );
	}
	
	function widget( $args, $instance ) {
		extract( $args );
		$title 		= apply_filters( 'widget_title', $instance['title'] );
		$slideshow 	= $instance['slideshow'];
		
		echo $before_widget;
		if ( $title ) {
			echo $before_title . $title . $after_title;
		}
		
		if ( '' != $slideshow ) {
			cnhk_slideshow( $slideshow );
		}
		echo $after_widget;
	}
	
	function form( $instance ) {
		$defaults = array( 'title' => '', 'slideshow' => '' );		
		$instance = wp_parse_args( ( array ) $instance, $defaults );
		$opt = get_option( 'cnhk_options' );
		if ( 0 < count( $opt[0] ) ) {
			echo '<p><label for="' . $this->get_field_id( 'title' ) . '">Title</label>
			<input type="text" class="widefat" id="' . $this->get_field_id( 'title' ) . '" name="' . $this->get_field_name( 'title' ) . '" value="' . $instance['title'] . '" /></p>';
			
			echo "<p><label for='" . $this->get_field_id( 'slideshow' ) . "'>Slideshow to display</label></p>";
			echo "<select name='" . $this->get_field_name( 'slideshow' ) . "' id='" .$this->get_field_id( 'slideshow' ) . "'>";
				echo "<option value=''"; 
				if ( !in_array( $instance['slideshow'], $opt[0] ) ) echo " selected='selected'";
				echo"></option>";
				for ( $i = 0; $i < count( $opt[0] ); $i++ ) {
					$selected = ( '' != $instance['slideshow'] && $instance['slideshow'] == $opt[0][$i] ) ? " selected='selected'" : '';
					echo "<option value='{$opt[0][$i]}'$selected>" . $opt[0][$i] . "</option>";
				}
			echo "</select>";
		} else {
			echo "<p class='error'>You need to create at least one slideshow. <a href='" . CNHK_ADMIN_SETTINGS . "'>Create one ?</a></p>";
		}
		
	}
	
	function update( $new_instance, $old_instance ) {
		$instance = $old_instance;
		$instance['title'] 		= strip_tags( $new_instance['title'] );
		$instance['slideshow'] 	= $new_instance['slideshow'];
		return $instance;
	}
	
}

/*
* Add Shortcode
*/

function cnhk_shortcode_fn( $atts, $content = null ) {
	extract( shortcode_atts( array( 'name' => '', 'width' => null, 'align' => null ), $atts ) );
	if ( ! empty( $width ) ) $width = intval( $width );
	if ( ! empty( $content ) ) {
		ob_start();
		cnhk_slideshow( $content, $width, $align );
		$output = ob_get_clean();
		return $output;
	} else {
		ob_start();
		cnhk_slideshow( $name, $width, $align );
		$output = ob_get_clean();
		return $output;
	}
}

add_shortcode( 'cnhk_slideshow', 'cnhk_shortcode_fn' );
